import traceback


def handle_exceptions(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            print(f"Exception occurred: {e}")
            print(f"Exception {traceback.format_exc()}")

    return wrapper


class BadStatusCode(Exception):
    def __init__(self, status_code, text='Error', message="Bad request"):
        self.status_code = status_code
        if status_code == 401:
            self.message = f"Check or update your IAM Token. Status code = {status_code}"
        if status_code == 404:
            self.message = f"{message}. Not found. Status code = {status_code}"
        else:
            self.message = f"{message}. Status code = {status_code}. Error message {text}"
        super().__init__(self.message)


class BadProjectRequest(Exception):
    def __init__(self, project_id, community_id):
        self.project_id = project_id
        self.community_id = community_id
        if self.project_id and self.community_id:
            self.message = 'Use only one of the available options'
        if (not self.project_id) and (not self.community_id):
            self.message = 'One of the parameters must be present'
        super().__init__(self.message)



import requests
import json
from etudatasphere.exceptions import BadProjectRequest, BadStatusCode
from etudatasphere.utils import parse_projects, ProjectSettings


def request_iam_token(oauth_token):
    url = "https://iam.api.cloud.yandex.net/iam/v1/tokens"
    headers = {"Authorization": f"OAuth {oauth_token}"}
    payload = {"yandexPassportOauthToken": oauth_token}

    res = requests.post(url, headers=headers, json=payload)

    if res.status_code != 200:
        raise BadStatusCode(res.status_code, res.text)
    else:
        response_json = res.json()
        return response_json.get("iamToken")


class DataSphereManager:

    def __init__(self, oauth_token: str) -> None:
        iam_token = request_iam_token(oauth_token)
        self.HEADERS = {"Authorization": "Bearer {}".format(iam_token)}

    def __make_get_request(self, url, params=None):
        res = requests.get(url, headers=self.HEADERS, params=params)
        if res.status_code != 200:
            raise BadStatusCode(res.status_code, res.text)
        else:
            return res

    def __make_post_request(self, url, data):
        res = requests.post(url, headers=self.HEADERS, data=json.dumps(data))
        if res.status_code != 200:
            raise BadStatusCode(res.status_code, res.text)
        else:
            return res

    def __make_patch_request(self, url, data):
        res = requests.patch(url, headers=self.HEADERS, data=json.dumps(data))
        if res.status_code != 200:
            raise BadStatusCode(res.status_code, res.text)
        else:
            return res

    def get_organizations(self):

        url = "https://resource-manager.api.cloud.yandex.net/resource-manager/v1/clouds"
        res = self.__make_get_request(url)
        orgs = res.json()['clouds']

        for org in orgs:
            print('NAME', org['name'])
            print('ID', org['organizationId'])
            print('*' * 25)

    def get_unit_balance(self, project_id):
        url = "https://datasphere.api.cloud.yandex.net/datasphere/v2/projects/{}:unitBalance".format(project_id)
        res = self.__make_get_request(url)
        return res.json()

    def get_billing_accounts(self):
        url = "https://billing.api.cloud.yandex.net/billing/v1/billingAccounts"
        res = self.__make_get_request(url)
        return res.json()

    def get_possible_ds_roles(self):
        url = "https://iam.api.cloud.yandex.net/iam/v1/roles"
        res = self.__make_get_request(url)
        roles = res.json()['roles']
        return [role for role in roles if 'datasphere' in role['id']]

    def get_organization_members(self, org_id: str):

        url = "https://organization-manager.api.cloud.yandex.net/organization-manager/v1/organizations/{}/users".format(
            org_id)
        res = self.__make_get_request(url)
        users = res.json()['users']
        for user in users:
            print('NAME', user['subjectClaims']['name'])
            print('ID', user['subjectClaims']['sub'])
            print('*' * 25)

    def get_organization_communities(self, organization_id: str):
        url = "https://datasphere.api.cloud.yandex.net/datasphere/v2/communities"
        params = {
            "organizationId": organization_id
        }
        res = self.__make_get_request(url, params)
        return res.json()

    def create_project(self,
                       community_id: str,
                       name: str,
                       description: str = None,
                       vmInactivityTimeout: str = "1800s"):
        data = ProjectSettings(
            community_id=community_id,
            name=name,
            description=description,
            vmInactivityTimeout=vmInactivityTimeout
        ).to_dict()
        url = "https://datasphere.api.cloud.yandex.net/datasphere/v2/projects"
        res = self.__make_post_request(url, data)
        return res.json()

    def check_operation_status(self, id_operation: str):
        url = "https://operation.api.cloud.yandex.net/operations/{}".format(id_operation)
        res = self.__make_get_request(url)
        return res.json()

    def get_projects(self, project_id: str = None, community_id: str = None, parse_projects_flag=False):
        if (community_id and project_id) or (
                (not project_id) and (not community_id)
        ):
            raise BadProjectRequest(community_id, project_id)

        if community_id:
            params = {
                "communityId": community_id
            }
        else:
            params = None
        url = "https://datasphere.api.cloud.yandex.net/datasphere/v2/projects/{}".format(
            project_id if project_id else "")
        res = self.__make_get_request(url, params)
        if parse_projects_flag:
            return parse_projects(res.json())
        else:
            return res.json()

    def add_contributors(self, project_id, contributors: list[dict]):
        """
        project_id: str - ID of project
        contributors: list[dict] - The list of contributors to add
            format:
                [
                    {
                    "id": "aje3e6pnq1io5fe801d3",
                    "role":"datasphere.communities.admin"
                    },
                    {
                    "id": "ajebni1dr6lgce3jkl3d",
                    "role": "datasphere.communities.admin"
                    }
                ]
        """

        if not contributors:
            return 'No participants selected'
        else:
            data = {'accessBindings': []}

            for user in contributors:
                data['accessBindings'].append({
                    "roleId": user['role'],
                    "subject": {
                        "id": user['id'],
                        "type": "userAccount"
                    }
                })
        url = "https://datasphere.api.cloud.yandex.net/datasphere/v2/projects/{}:setAccessBindings". \
            format(project_id)

        res = self.__make_post_request(url, data)
        return res.json()

    def update_unit_balance(self, project_id, new_unit_balance):
        url = "https://datasphere.api.cloud.yandex.net/datasphere/v2/projects/{}:unitBalance".format(project_id)
        data = {"unitBalance": new_unit_balance}
        res = self.__make_post_request(url, data)
        return res.json()

    def update_all_community_projects(self,
                                      community_id: str,
                                      vmInactivityTimeout: str = "1000s",
                                      unit_balance: int = None):
        data = ProjectSettings(
            vmInactivityTimeout=vmInactivityTimeout
        ).to_dict()

        projects = self.get_projects(community_id=community_id)

        updateMask_list = []
        for k, v in data.items():
            for n in v.keys():
                updateMask_list.append(f"{k}.{n}")
        data['updateMask'] = ', '.join(updateMask_list)

        try:
            for idx, project in enumerate(projects['projects']):
                print(f"{idx + 1}/{len(projects['projects'])}")
                url = "https://datasphere.api.cloud.yandex.net/datasphere/v2/projects/{}".format(project.get('id'))
                res = self.__make_patch_request(url, data)
                print('Operation id:', res.json()['id'])

                if unit_balance is not None:
                    self.update_unit_balance(project.get('id'), unit_balance)

            return 'ok'
        except Exception as e:
            raise e



import pytest
from etudatasphere import DataSphereManager


@pytest.fixture(scope="module")
def data_sphere_manager():
    data_sphere_manager = DataSphereManager(oauth_token="")
    return data_sphere_manager


def test_get_organizations(data_sphere_manager):
    organizations = data_sphere_manager.get_organizations()


def test_get_communities(data_sphere_manager):
    organization_id = "bpfa94oocmi345p7tpv7"
    communities = data_sphere_manager.get_organization_communities(organization_id)
    print(communities)


def test_get_projects(data_sphere_manager):
    community_id = "bt1u8sr3l01444sc22gm"
    projects = data_sphere_manager.get_projects(community_id=community_id)
    print(projects)


def test_get_unit_balance(data_sphere_manager):
    project_id = "bt15an734pcprskkvjkt"
    print(data_sphere_manager.get_unit_balance(project_id))


def test_update_unit_balance(data_sphere_manager):
    project_id = "bt15an734pcprskkvjkt"
    data_sphere_manager.update_unit_balance(project_id, 100)


def test_create_project(data_sphere_manager):
    community_id = "bt1u8sr3l01444sc22gm"
    name = "Test Project"
    description = "This is a test project"
    vm_inactivity_timeout = "1800s"
    data_sphere_manager.create_project(community_id, name, description, vm_inactivity_timeout)


def test_update_all_community_projects(data_sphere_manager):
    community_id = "bt1u8sr3l01444sc22gm"
    vm_inactivity_timeout = "666s"
    unit_balance = 150
    data_sphere_manager.update_all_community_projects(community_id, vm_inactivity_timeout, unit_balance)



from io import open
from setuptools import setup

version = '0.1.2'

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='etudatasphere',
    version=version,

    author='sesh00',
    author_email='ernestrsage@gmail.com',

    description=(
        u'etudatasphere: Simplifying Cloud Administration. Access Yandex.Cloud services with ease, automating routine '
        u'tasks for resource management.'
    ),
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/sesh00/etudatasphere',
    download_url='https://github.com/sesh00/etudatasphere/archive/main.zip'.format(
        version
    ),

    license='MIT License, see LICENSE file',

    packages=['etudatasphere'],
    install_requires=['requests'],

    extras_require={
            'dev': [
                'pytest',
            ]
        },

    classifiers=[
        'Operating System :: OS Independent',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
    ]
)



def parse_projects(res: dict):
    if res.get('projects'):
        parse_str = """"""
        for project in res.get('projects'):
            parse_str = parse_str + f"""
Project name: {project.get('name')}\n
Project ID: {project.get('id')}\n
Project settings:\n
{parse_settings(project.get('settings'))}\n
***********************************************
        """
        return parse_str
    else:
        return f"""
Project name: {res.get('name')}\n
Project ID: {res.get('id')}\n
Project settings:\n
{parse_settings(res.get('settings'))}\n
***********************************************
        """


def parse_settings(project_settings: dict):
    settings_str = ""
    for el in project_settings:
        settings_str += f"\t{el}:{project_settings[el]}\n"
    return settings_str


class ProjectSettings:
    settings = ('vmInactivityTimeout')
    limits = ('maxUnitsPerHour', 'maxUnitsPerExecution')

    def __init__(self, community_id: str = None,
                 name: str = None,
                 description: str = None,
                 vmInactivityTimeout: str = "300s"):
        if community_id:
            self.community_id = community_id
        if name:
            self.name = name
        if description:
            self.description = description
        if vmInactivityTimeout:
            self.vmInactivityTimeout = vmInactivityTimeout

    def to_dict(self):
        base_dict = {}
        settings_dict = {}
        limits_dict = {}
        for key, value in self.__dict__.items():
            if not (key.startswith('_') or key.endswith('__') or callable(
                    key)):
                if key in self.settings:
                    settings_dict[key] = value
                elif key in self.limits:
                    limits_dict[key] = value
                else:
                    base_dict[key] = value
        if settings_dict:
            base_dict['settings'] = settings_dict
        if limits_dict:
            base_dict['limits'] = limits_dict
        return base_dict
